
;******************************************************************************
;******************************************************************************
;ds18s20 routinen
;******************************************************************************

reset_pulse:
	sbi ddrb, 0				;pd0 als ausgang
	cbi portb, 0	
	rcall wait500us				;mcu hlt bus fr mindestens 480us low
	cbi ddrb, 0				;lsst den bus los, geht in rx-mode
	rcall wait70us				;wartet 70 us
	
	wait_presence_pulse:
	sbic pinb, 0
	rjmp wait_presence_pulse		;wartet auf presence pulse
	
	wait_presence_pulse_end:
	sbis pinb, 0				
	rjmp wait_presence_pulse_end 		;wartet bis presence pulse fertig

	rcall wait410us				;wartet 412 us

ret

;******************************************************************************

write_1:					;schreibt eine 1
	
	sbi ddrb, 0				;PB0 Ausgang
	cbi portb, 0				;bus auf low
	rcall wait6us				;wartet 8us (maximum sind 15us)
	cbi ddrb, 0				;lsst bus los, nun durch ext R auf high
	rcall wait64us				;slot muss mind. 60us lang sein
	ret

;******************************************************************************
write_0:					;schreibt eine 0
	
	sbi ddrb, 0				;PB0 ausgang
	cbi portb, 0			;bus auf low
	rcall wait60us
	cbi ddrb, 0				;lsst bus los, nun durch ext R auf high
	rcall wait10us				;slot muss mind. 60us lang sein

	ret

;******************************************************************************
write_command:					;gibt einen befehl aus, lsb zuerst

	sbrs befehl, 0	
	rcall write_0
	sbrc befehl, 0
	rcall write_1
	
	ror befehl				;nchstes bit
	ldi temp, 7

	write_command_1:
	sbrs befehl, 0	
	rcall write_0
	sbrc befehl, 0
	rcall write_1
	ror befehl
	dec temp
	brne write_command_1

ret

;******************************************************************************
skip_rom:					;gibt den befehl "Skip Rom" an den
	ldi temp, 0xCC				; sensor aus
	mov befehl, temp
	rcall write_command
ret

;******************************************************************************
convert_t:					;gibt den befehl "Convert T" an den
	ldi temp, 0x44				; sensor aus
	mov befehl, temp
	rcall write_command			

	cbi portb, 0
	sbi ddrb, 0				;bus low fr 6us
	rcall wait6us
	cbi ddrb, 0				;bus loslassen
	rcall wait9us				;nach 9us samplen
	convert_t_1:
	sbis pinb, 0				;wartet bis conversion zu ende ist
	rjmp convert_t_1
	
ret

;******************************************************************************
read_scratchpad:				;gibt den befehl "Read Scratchpad" 
	ldi temp, 0xBE				; an den sensor aus
	mov befehl, temp
	rcall write_command

	rcall read_bit				;nach dem befehl werden alle 9 byte
	sts temperature_lsb, temp2		; geschickt und eingelesen

	rcall read_bit
	sts temperature_msb, temp2


	ret



	;*****************************************************
	read_bit:
	ldi temp2, 0				;temp2 auf 0
	ldi temp, 8				;8mal durchfhren

	read_bit_0:
	ror temp2
	cbi portb, 0
	sbi ddrb, 0				;bus low fr 6us
	rcall wait6us
	cbi ddrb, 0				;bus loslassen
	rcall wait9us				;nach 9us samplen

	sbis pinb, 0				;schaut nach ob nun eine 0 anliegt
	rjmp read_bit_1				;wenn ja, springe weiter
	ldi temp1, 128
;add temp2, r26
	add temp2, temp1
	rjmp read_bit_2

	read_bit_1:				
	ldi temp1, 0
	add temp2, temp1

	read_bit_2:

	rcall wait55us
	dec temp
	brne read_bit_0				;zum schluss steht das fertige byte in tmp3
ret
