;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                 LCD-Routinen             ;;
;;                 ============             ;;
;;                                          ;;
;; 4bit-Interface                           ;;
;; DB4-DB7:       PD0-PD3                   ;;
;; RS:            PD4                       ;;
;; E:             PD5                       ;;
;; R/W:           an Masse schlieen        ;;
;; es werden folgende Register benoetigt    ;;
;; Dateneingabe: temp Hilfsregister: temp1  ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

 
;**********  Initialisierung*********************************************************************
lcd_init:
	ldi	temp,0b00000011		;muss 3mal hintereinander gesendet
	out	PORTD,temp		;werden zur Initialisierung
	rcall	lcd_enable		;1
	rcall	delay5ms
	rcall	lcd_enable		;2
	rcall	delay5ms
	rcall	lcd_enable		;3
	rcall	delay5ms

	ldi	temp,0b00100000		;set Function (zuerst mal 4 bit Interface aktivieren)
	rcall	lcd_command		;sonst komme ich nmlich nicht an Bit 3 ran (fr Modus 2-zeilig)
	rcall	delay5ms

	ldi	temp,0b00101000		;set Function (2 zeilig, Font 5x7)
	rcall	lcd_command
	rcall	delay5ms

	ldi	temp,0b00001100		;Display on/off Control (Display on, Cursor off, Blinking off)
	rcall	lcd_command
	rcall	delay5ms

	ldi	temp,0b00000100		;set Entry Mode (Cursor Direction right, Display shift off)
	rcall	lcd_command
	rcall	delay5ms

	ret				;fertig

;**********  Sendet den Befehl zur Lschung des Displays  *******************************************
lcd_clear:
	ldi	temp, 0b00000001	;Display lschen
	rcall	lcd_command
	rcall	delay5ms
	ret
 
;**********  sendet ein Datenbyte an das LCD  *******************************************************
lcd_data:
    push temp1			; Daten von Temp auf dem Stack sichern
	mov	temp1,temp		;Daten sichern
	swap	temp			;oberes Nibble runterholen
	andi	temp,0b00001111		;4 MSB auf Null setzen
	ori	temp,0b00010000		;RS setzen
	out	PORTD,temp		;ursprnglich oberes Nibble ausgeben
	rcall	lcd_enable		;Enable
	andi	temp1,0b00001111	;obere 4 Bit auf Null setzen 
	ori	temp1,0b00010000	;RS setzen
	out	PORTD,temp1		;unteres Nibble ausgeben
	rcall	lcd_enable		;Enable
    rcall	delay50us		;warte kurz
	pop temp1		; Daten vom Stack holen
	ret				;fertig

;**********  sendet einen Befehl an das LCD  ******************************************************
lcd_command:				;wie lcd_data, nur ohne RS zu setzen
	push temp1
	mov	temp1,temp
	swap	temp
	andi	temp,0b00001111
	out	PORTD,temp
	rcall	lcd_enable
	andi	temp1,0b00001111
	out	PORTD,temp1
	rcall	lcd_enable
	rcall	delay50us
	pop temp1
	ret

;********** Eine 8 bit Zahl ohne Vorzeichen aus dem Register temp dezimal ausgeben  **********************************
;********** Nullen werden unterdrckt *******************************************************
lcd_number:
           push  temp
           push  temp1
          
           mov   temp1, temp
		                          ; abzhlen wieviele Hunderter
								  ; in der Zahl enthalten sind
		   ldi   temp, '0'
lcd_number_1:
           subi  temp1, 100
           brcs  lcd_number_2
           inc   temp
           rjmp  lcd_number_1
                                          ;
lcd_number_2:
			cpi temp,'0'
			breq lcd_number_2a    
			rcall lcd_data                ; die Hunderterstelle ausgeben wenn nicht 0
lcd_number_2a:
           subi  temp1, -100              ; 100 wieder dazuzhlen, da die
                                          ; vorherhgehende Schleife 100 zuviel
                                          ; abgezogen hat

                                          ; abzhlen wieviele Zehner in
                                          ; der Zahl enthalten sind
           ldi   temp, '0'
lcd_number_3:
           subi  temp1, 10
           brcs  lcd_number_4
           inc   temp
           rjmp  lcd_number_3

lcd_number_4:
			cpi temp,'0'
			breq lcd_number_4a    
			rcall lcd_data                ; die Zehnerstelle ausgeben wenn nicht 0
lcd_number_4a:
           subi  temp1, -10               ; 10 wieder dazuzhlen, da die
                                          ; vorhergehende Schleife 10 zuviel
                                          ; abgezogen hat

                                          ; die brig gebliebenen Einer
                                          ; noch ausgeben
           ldi   temp, '0'
           add   temp, temp1
           rcall lcd_data

           pop   temp1
           pop   temp
           ret



;**********  erzeugt den Enable-Puls  **************************************************************
lcd_enable:
	sbi	PORTD,5			;Enable auf high
	nop				;3 Taktzyklen warten
	nop
	nop
	cbi	PORTD,5			;Enable wieder auf low
	ret				;ferig

;**********  Sendet den Befehl zum Einstellen eigener Sonderzeichen  ******************************



;**********  Pause nach jeder bertragung  *******************************************************
;**********  fr eine Quarzfrequenz von 3,2768 MHz  **********************************************
;**********  Werte in Klammern fr 4 MHz  ********************************************************
delay50us:				;50us Pause
	ldi	temp,41			;(50)
delay50us_1:
	nop
	dec	temp
	brne	delay50us_1
	ret				;fertig

;**********  Lngere Pause fr manche Befehle  **************************************************
delay5ms:				;5ms Pause
	ldi	temp,21			;(25)
delay5ms_1:
	ldi	temp1,195		;(200)
	nop
delay5ms_2:
	dec	temp1
	nop
	brne	delay5ms_2
	dec	temp
	brne	delay5ms_1
	ret				;fertig
